{
  // requires insta_nonce, user_id, is_frontend via php

  // Backend(Profile page): Scans page for `almefy-send-mail` buttons and adds notice on mail send
  async function setup() {
    const { user_id, insta_nonce } = wpv_mail;
    const { __ } = wp.i18n;

    const send_mail_buttons = document.querySelectorAll(
      ".almefy-send-mail"
    );

    for (const button of send_mail_buttons) {
      button.addEventListener("click", async (event) => {
        event.preventDefault();

        const success_notification = document.querySelector(
            ".almefy-success-notification > p"
          );
        const error_notification = document.querySelector(
            ".almefy-error-notification"
          );

        success_notification.style.display = 'none';
        error_notification.style.display = 'none';
        
        const old_label = button.innerText;
        button.innerText = "...";
        const response = await fetch(almefy_api + "device/add", {
          method: "post",
          body: JSON.stringify({ user_id }),
          headers: {
            "Content-Type": "application/json",
            "X-WP-NONCE": insta_nonce,
          },
        });

        button.innerText = old_label;


        // const info_area = document.querySelector(".almefy-info-area");
        if (response.ok) {
          const json = await response.json();

          /** @type{HTMLDivElement} */

          success_notification.style.display = "block";

          success_notification.innerHTML = `<p>${__(
            "Email has been sent to",
            "almefy-me"
          )} <b>${json.sent_to}</b> !</p>`;

          // info_area.appendChild(box);
        } else {
          const body = await response.text();

          /** @type{HTMLDivElement} */
          error_notification.style.display = "block";

          console.error("almefy: could not send mail", response);
          error_notification.innerHTML = `${__(
            "Email could not be sent!",
            "almefy-me"
          )}!<br> ${__(
            "Please try again later or contact an administrator",
            "almefy-me"
          )}.<br>${body}`;

          console.error(response);
        }
      });
    }
  }

  setup();
}
