{
  // script requires insta_nonce, is_frontend, show_activation provided by php
  // const { __ } = wp.i18n;

  // Load devices and hide activation or management options
  async function setup() {
    if (!wpv.is_frontend) draw_table_backend();

    const refreshers = document.querySelectorAll(".almefy-refresh");
    const activators = document.querySelectorAll(".almefy-activate");

    for (const refresh of refreshers) {
      refresh.addEventListener("click", async (event) => {
        event.preventDefault();
        let old_label = refresh.innerText;
        refresh.innerText = "...";

        await fetch_devices();
        refresh.innerText = old_label;
      });
    }

    const devices = await fetch_devices();
    if (!wpv.is_frontend) {
      if (devices && devices.length > 0 && wpv.show_activation !== "1") {
        for (const activate of activators) {
          activate.style.display = "none";
        }
      } else {
        for (const refresh of refreshers) {
          // refresh.style.display = "none";
        }
      }
    }

    // add device button

    if (wpv.is_frontend) {
      /** @type{HTMLDivElement} */
      const modal = document.createElement("div");
      modal.classList.add("almefy-modal");
      modal.innerHTML = `
        <span class="almefy-modal-text">
          ${wp.i18n.__("An email will be sent to", "almefy-me")} <br>
          <b>${wpv.email}</b>
          <br>${wp.i18n.__("Are you sure?", "almefy-me")}
        </span>  
        <div style="margin-top: 1rem;">
          <button class="almefy-modal-button no almefy-device-button">${wp.i18n.__(
            "Cancel",
            "almefy-me"
          )}</button>
          <button class="almefy-modal-button yes almefy-device-button">${wp.i18n.__(
            "Send Mail",
            "almefy-me"
          )}</button>
       </div>
     `;

      const modal_button_no = modal.querySelector(".no");
      const modal_button_yes = modal.querySelector(".yes");

      modal_button_no.addEventListener("click", async (event) => {
        event.preventDefault();
        modal.classList.remove("active");
      });

      modal_button_yes.addEventListener("click", async (event) => {
        event.preventDefault();

        const old_label = modal_button_yes.innerText;
        modal_button_yes.innerText = "...";
        const response = await fetch(almefy_api + "device/add", {
          method: "post",
          body: JSON.stringify({ user_id: wpv.user_id }),
          headers: {
            "Content-Type": "application/json",
            "X-WP-NONCE": wpv.insta_nonce,
          },
        });

        modal_button_yes.innerText = old_label;
        modal_button_yes.style.display = "none";
        modal_button_no.innerText = "Close";

        const info_area = modal.querySelector(".almefy-modal-text");
        if (response.ok) {
          const json = await response.json();
          // info_area.innerText = wp.i18n.__(`Email has been sent to ${json.sent_to} !`, 'almefy');
          info_area.innerHTML = `<p>${wp.i18n.__(
            "Email has been sent!",
            "almefy-me"
          )}</p>`;
        } else {
          const body = await response.text();
          // info_area.innerText = wp.i18n.__(`Email has been sent to ${json.sent_to} !`, 'almefy');
          info_area.innerHTML = `${wp.i18n.__(
            "Email could not be sent!",
            "almefy-me"
          )}<br>${wp.i18n.__(
            "Please try again later or contact an administrator.",
            "almefy-me"
          )}<br>${body}`;
          console.error("almefy: could not send mail.", response);
        }

        // modal.classList.remove("active");
      });

      const devices_container = document.querySelector(".almefy-devices");

      devices_container.appendChild(modal);

      const add_device_buttons = document.querySelectorAll(
        ".almefy-add-device"
      );

      for (const button of add_device_buttons) {
        button.addEventListener("click", (event) => {
          event.preventDefault();
          modal.classList.add("active");
        });
      }
    }
  }

  function draw_table_backend() {
    const device_container = document.querySelector(
      ".almefy-devices-admin"
    );
    if (device_container) {
      // TODO: translate
      device_container.innerHTML = `
      <div>
        <table class="widefat" role="presentation">
            <thead>
                <tr>
                    <th class="row-title">Label</th>
                    <th class="row-title">Device</th>
                    <th class="row-title">Created at</th>
                </tr>
            </thead>

            <tbody class="almefy-device-table">
            </tbody>

        </table>

        <div style="display:none;" class="almefy-success-notification"><p></p></div>
        <div style="display:none;" class="almefy-error-notification"><p></p></div>

        <button class="almefy-button almefy-send-mail">${wp.i18n.__(
          "Add Device",
          "almefy-me"
        )}</button>
        <button class="almefy-button almefy-refresh">${wp.i18n.__(
          "Refresh",
          "almefy-me"
        )}</button>
    </div>`;
    }
  }

  // (Re)build list HTML
  function draw_devices_backend(devices) {
    const device_table = document.querySelector(".almefy-device-table");
    // const device_list = document.querySelector(".almefy-device-list");
    if (device_table) {
      device_table.innerHTML = "";

      for (const device of devices) {
        const tr = document.createElement("tr");

        const label = document.createElement("td");
        label.innerText = device.label;

        const model = document.createElement("td");
        model.innerText = device.model;

        const created_at = document.createElement("td");
        const date = new Date(device.created_at);
        created_at.innerText = `${date.toLocaleTimeString()} - ${date.toLocaleDateString()}`;

        const delete_td = document.createElement("td");
        delete_td.style.width = '150px';

        const delete_button = document.createElement("a");
        delete_button.href = "#";
        delete_button.classList.add("almefy-delete");
        delete_button.innerText = "Delete";

        delete_td.appendChild(delete_button);

        tr.appendChild(label);
        tr.appendChild(model);
        tr.appendChild(created_at);
        tr.appendChild(delete_td);

        let clicked_once = false;

        delete_button.addEventListener("click", async (event) => {
          event.preventDefault();

          if (!clicked_once) {
            clicked_once = true;
            delete_button.innerText = wp.i18n.__("Confirm deletion?", "almefy-me");
            return;
          }

          delete_button.innerText = "...";
          delete_button.disabled = true;

          console.log("almefy - deleting device");

          if (await delete_device(device.id)) {
            await fetch_devices();
          } else {
            // TODO: Show error
          }
        });

        device_table.appendChild(tr);
      }
    }
  }

  // (Re)build list HTML
  function draw_devices_frontend(devices) {
    const device_lists = document.querySelectorAll(".almefy-device-list");

    for (const device_list of device_lists) {
      device_list.innerHTML = "";

      if (devices.length == 0) {
        device_list.innerHTML = wp.i18n.__("No devices connected.", "almefy-me");
      }

      for (const device of devices) {
        const li = document.createElement("li");
        li.innerHTML = `
          <div class="almefy-device-entry">
            <span class='almefy-device-label'>${device.label}</span>
            <small class='almefy-device-model'>${device.model}</small>
          </div>
        `;

        // Delete Button
        /** @type{HTMLButtonElement} */
        const delete_button = document.createElement("button");
        delete_button.innerHTML = `<svg class="svg-inline--fa fa-times fa-w-10" data-prefix="far" data-icon="times" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512" data-fa-i2svg=""><path fill="currentColor" d="M207.6 256l107.72-107.72c6.23-6.23 6.23-16.34 0-22.58l-25.03-25.03c-6.23-6.23-16.34-6.23-22.58 0L160 208.4 52.28 100.68c-6.23-6.23-16.34-6.23-22.58 0L4.68 125.7c-6.23 6.23-6.23 16.34 0 22.58L112.4 256 4.68 363.72c-6.23 6.23-6.23 16.34 0 22.58l25.03 25.03c6.23 6.23 16.34 6.23 22.58 0L160 303.6l107.72 107.72c6.23 6.23 16.34 6.23 22.58 0l25.03-25.03c6.23-6.23 6.23-16.34 0-22.58L207.6 256z"></path></svg>`;

        // Modal

        /** @type{HTMLDivElement} */
        const delete_modal = document.createElement("div");
        delete_modal.classList.add("almefy-modal");
        delete_modal.innerHTML = `
          <b>Remove device?</b><br>
          ${device.model}
          <div style="margin-top: 1rem;">
            <button class="almefy-modal-button no almefy-device-button">${wp.i18n.__(
              "No",
              "almefy-me"
            )}</button>
            <button class="almefy-modal-button yes almefy-device-button">${wp.i18n.__(
              "Yes",
              "almefy-me"
            )}</button>
          </div>
        `;

        delete_button.addEventListener("click", (event) => {
          event.preventDefault();

          delete_modal.classList.add("active");
        });

        const modal_button_no = delete_modal.querySelector(".no");
        const modal_button_yes = delete_modal.querySelector(".yes");

        modal_button_no.addEventListener("click", async (event) => {
          event.preventDefault();
          delete_modal.classList.remove("active");
        });

        modal_button_yes.addEventListener("click", async (event) => {
          event.preventDefault();

          if (await delete_device(device.id)) {
            await fetch_devices();
          } else {
            // TODO: Show error
            console.error("Could not remove device.");
          }

          delete_modal.classList.remove("active");
        });

        li.appendChild(delete_button);
        device_list.appendChild(li);
        device_list.appendChild(delete_modal);
      }
    }
  }

  // API call to delete a device
  async function delete_device(id) {
    const response = await fetch(almefy_api + "device/remove", {
      method: "post",
      body: JSON.stringify({ device_id: id }),
      headers: {
        "Content-Type": "application/json",
        "X-WP-NONCE": wpv.insta_nonce,
      },
    });

    return response.ok;
  }

  // API call to fetch enabled devices
  async function fetch_devices() {
    const response = await fetch(almefy_api + "devices", {
      method: "get",
      headers: { "X-WP-NONCE": wpv.insta_nonce },
    });

    if (response.ok) {
      const json = await response.json();
      if (wpv.is_frontend) draw_devices_frontend(json);
      else draw_devices_backend(json);
      return json;
    } else {
      // TODO: show error message
    }
  }

  //
  setup();
}
