<?php

if (!defined('ABSPATH')) {
    exit;
}

class AlmefyConnectDeviceShortcode {

    public function __construct() {
        $this->shortcode();
    }

    private function shortcode() {

        add_shortcode('almefy-connect', function($attributes = [], $content = null) {
            $attributes = shortcode_atts([], $attributes, 'almefy-connect');

            if(!is_user_logged_in()) {
                return "<div class='almefy-devices-not-logged-in'>" . __('You must be logged in to connect devices.', 'almefy-me') . "</div>";
            }
            
            $user = wp_get_current_user();
            $email = $user->user_email;

            $enrollment_token = null;
            try {
                $enrollment_token = AlmefyManager::$client->enrollIdentity($email, ['sendEmail' => false]);
            } catch (\Throwable $th) {
                return "<div style='margin: auto;'>" . __("Could not create qr code. Please try again later.", 'almefy-me') . "</div>";
            }

            $img = $enrollment_token->getBase64ImageData();
            $src = "data:image/png;base64,$img";
            
            $token_id = $enrollment_token->getId();
            $connect_url = "https://app.almefy.com/?c=$token_id";

            ob_start(); ?>
            
                <div class="almefy-me-connect">
                    <div class="flex flex-col">
                        
                        <div class="connect-desktop">
                            <h3 style=""><?php _e("Connect Device", "almefy-me") ?></h3>
                            <img width="120px" height="120px" id='enrollment-code' class='qr blurred' width="120px" src="<?php echo $src ?>">
                            
                            <p class="almefy-me-scan-me"><?php _e("Scan with Almefy app.", "almefy-me") ?></p>
                            <button id='show-enroll' class="button"><?php _e("Show Code", "almefy-me") ?></button>
                        </div>
                        
                        <div class="connect-mobile">
                            <h3 style=""><?php _e("Connect Device", "almefy-me") ?></h3>
                            <a href="<?php echo esc_url($connect_url) ?>">
                                <button class="button almefy-connect-button"><?php _e('Connect Device', 'almefy-me') ?></button>
                            </a>
                        </div>                       
    
                        <a href="#" style="text-align: center; margin-top: 1rem; color: #2D4D74;" id='send-mail'><?php echo sprintf( __( 'Send mail to %s instead.' ) , $email ) ?></a>
                    </div>
                </div>


                <script>
                    {
                        
                        // enrollment button
                        {

                            const button = document.querySelector('#show-enroll');
                            let hidden = true;
                            const qr = document.querySelector('#enrollment-code');
                            const old_text = button.innerText

                            // cancel timer when user hides code manually
                            let active_timer = null;

                            button.addEventListener('click', async (e) => {
                                e.preventDefault();
                                hidden = !hidden;

                                // TODO: request new image
                                
                                if(hidden) {
                                    qr.classList.add('blurred');
                                    button.innerText = old_text;
                                    clearTimeout(active_timer);
                                    active_timer = null;
                                    qr.src = await fetch_qr();
                                }
                                else  {
                                    qr.classList.remove('blurred');
                                    button.innerText = 'Hide Code';

                                    active_timer = setTimeout(async () => {
                                        // TODO: visual timer
                                        qr.classList.add('blurred');
                                        hidden = true;
                                        button.innerText = old_text;
                                        // if redraw_list {
                                        //     await redraw_list();
                                        // }

                                        qr.dispatchEvent(new Event("almefyQrHidden"));
                                        
                                        qr.src = await fetch_qr();

                                    }, 5 * 1000);
                                }

                            
                            });
                        }

                        async function fetch_qr() {

                            const rest = "<?php echo rest_url('almefy/v1/') ?>";
                            const nonce = "<?php echo wp_create_nonce('wp_rest') ?>";
                            const user_id = "<?php echo get_current_user_id(); ?>";

                            const response = await fetch(rest + "device/connect_qr", {
                                method: "get",
                                headers: { "X-WP-NONCE": nonce },
                            });
    
                            if (response.ok) {
                                const text = await response.text();
                                // Undo escaping done by wordpress
                                return JSON.parse(text);
                            } else {
                                // TODO: show error in frontend
                                console.error('almefy - could not fetch qr image.')
                                return '';
                            }
                        }

                        // send enrollment mail button
                        {

                            const rest = "<?php echo rest_url('almefy/v1/') ?>";
                            const nonce = "<?php echo wp_create_nonce('wp_rest') ?>";
                            const user_id = "<?php echo get_current_user_id(); ?>";
                            const email = "<?php echo $email ?>";

                            const button = document.querySelector('#send-mail');
                            button.addEventListener('click', async (e) => {
                                e.preventDefault();

                                const old_text = button.innerText;
                                button.innerText = '...';
                                button.disabled = true;

                                const response = await fetch(rest + "device/add", {
                                    method: "post",
                                    body: JSON.stringify({ user_id }),
                                    headers: {
                                        "Content-Type": "application/json",
                                        "X-WP-NONCE": nonce,
                                    },
                                });

                                if (response.ok) {
                                    // show mail sent
                                    alert(`Mail has been sent to ${email}.`);
                                } else {
                                    console.error('almefy - enrollment mail could not be sent.')
                                }

                                button.disabled = false;
                                button.innerText = old_text;
                            })
                        }
                    }
                </script>
            <?php
            return ob_get_clean();
        });
    }

}