<?php

if (!defined('ABSPATH')) {
    exit;
}

class AlmefyLoginShortcode
{
    public function __construct()
    {
        add_shortcode('almefy-login', function ($attributes = [], $content = null) {

            // API disabled via settings?
            $api_enabled = get_option('almefy-api-enabled');
            if ($api_enabled != 1) {
                return '';
            }

            // SETTINGS
            $attributes = shortcode_atts([
                'show_when_logged_in' => "false",
                'redirect' => '', 
                'login_page' => 'false',
            ], $attributes, 'almefy-login');

            $showWhenLoggedIn = $attributes['show_when_logged_in'] == 'true';
            $redirect = $attributes['redirect'];
            $login_page = $attributes['login_page'] == 'true';

            if (!$showWhenLoggedIn && is_user_logged_in()) {
                return '';
            }

            // RENDER
            if (!$login_page) {
                return $this->html($redirect);
            } else {
                return $this->login_page_html($redirect);
            }
        });
    }

    // We can't insert HTML into the login page regularly, so we must use js.
    public function login_page_html($redirect) {
        ob_start();
        ?>

        <script>
            /** @type {HTMLButtonElement} */
            let submit = document.querySelector("p.submit");

            if (submit) {
                const div = document.createElement("div");
                div.classList.add('almefy-login-page-container');
                div.innerHTML += `
                    <?php echo $this->html($redirect) ?>
                `;
                submit.after(div);
            } else {
                console.error("Almefy: Could not add qr code login page. Submit button not found to attach to.")
            }
        </script>

        <?php
        return ob_get_clean();
    }

    // HTML & CSS form qr code display
    public function html($redirect)
    {
        $api_key = get_option('almefy-api-key');
        $auth_url = rest_url("almefy/v1/login-controller");

        if($redirect == 'stayonpage') {
            global $wp;
            $redirect = home_url( add_query_arg( array(), $wp->request ) );
        }

        // append "redirect" parameter to plain wordpress permalinks. 
        if(strpos($auth_url, "rest_route=") !== false) {
            $auth_url .= "&redirect=$redirect";
        } else {
            $auth_url .= "?redirect=$redirect";
        }

        ob_start(); ?>
        <div class="almefy-card">
            <div data-almefy-auth
                data-almefy-api="<?php echo esc_attr(AlmefyManager::$api_url) ?>"
                data-almefy-key="<?php echo esc_attr($api_key) ?>"
                data-almefy-auth-url="<?php echo esc_url($auth_url) ?>">
            </div>
        </div>
<?php

        return ob_get_clean();
    }
}
